# encoding: utf-8
from abc import ABCMeta


class BaseItem:
    """Base check item class."""

    __metaclass__ = ABCMeta

    def __init__(self, json_obj, patient_name, expected_date):
        self.json_obj = json_obj
        self.data = json_obj['data']
        self.identification = json_obj.get('identification', u'空')
        self.patient_name = patient_name
        self.expected_date = expected_date

    def is_valid(self):
        """check whether the data is valid."""
        return True



# encoding: utf-8
import argparse

from datetime import datetime

from handler import handler_check_dynamic_treats, handler_change_enter_date, handle_check_all

DATE_FORMAT = '%Y-%m-%d'


def valid_date(s):
    try:
        t = datetime.strptime(s, DATE_FORMAT)
    except ValueError:
        msg = "Not a valid date: '{0}'.".format(s)
        raise argparse.ArgumentTypeError(msg)

    return datetime.strftime(t, DATE_FORMAT)


def get_parser():
    """get command parser."""
    parser = argparse.ArgumentParser(
        description="check and fix errors in NDIP", )
    parser.add_argument("-o", "--host",
                        default="localhost",
                        help="mysql host, default localhost")
    parser.add_argument("-u", "--user",
                        default="root",
                        help="mysql user, default root")
    parser.add_argument("-p", "--password",
                        default="ChangeMe1!",
                        help="mysql password, default ChangeMe1!")
    parser.add_argument("-d", "--database",
                        default="hospitalAdmin",
                        help="mysql database, default hospitalAdmin")

    subparsers = parser.add_subparsers(help='operations')

    # check all treatments
    check_all_parser = subparsers.add_parser('all', help='check all treatments')
    check_all_parser.set_defaults(func=handle_check_all)

    # check dynamic treatments
    dynamic_treat_parser = subparsers.add_parser(
        'dynamic-treat', help='check dynamic treatments')
    dynamic_treat_parser.add_argument(
        '-f', '--fix', action='store_true',
        help='fix error founded in dynamic treatments')
    dynamic_treat_parser.set_defaults(func=handler_check_dynamic_treats)

    # change enter date
    change_enter_date_parser = subparsers.add_parser(
        'change-enter-date', help="change patient's enter date")
    change_enter_date_parser.add_argument(
        'record_id', action='store', type=int,
        help='patient treatment record ID')
    change_enter_date_parser.add_argument(
        'original', action='store', type=valid_date,
        help='original enter date - format YYYY-MM-DD')
    change_enter_date_parser.add_argument(
        'target', action='store', type=valid_date,
        help='target enter date - format YYYY-MM-DD')
    change_enter_date_parser.set_defaults(func=handler_change_enter_date)

    return parser


def parse_args(args):
    """return parsed args"""

    parser = get_parser()
    return parser.parse_args(args)



# encoding: utf-8
import json

from datetime import datetime, timedelta

from items import ITEMS, check, command_confirm
from patient import update_patient_enter_date

KEY_TREAT_LIST = 'treatlist'
KEY_CHECKS = 'form'
KEY_TAG_DATE = 'tagDate'
KEY_IS_STARTED = 'is_started'
KEY_EXPECT_DATE = 'expectdate'
KEY_ACTUAL_DATE = 'actualdate'

VALUE_DYNAMIC_ADD = u'动态增加'

DATE_FORMAT = '%Y-%m-%d'


class Treat:
    """Represents a patient's one treatment record."""

    def __init__(self, json_obj, patient_name='', fix=False):
        self.data = json_obj
        self.patient_name = patient_name
        self.expected = self.data[KEY_EXPECT_DATE]
        self.fix = fix

    def check(self):
        """return value is whether treatment has did at least once fix."""

        # Only check Dynamic added treatment.
        try:
            if self.data[KEY_TAG_DATE] != VALUE_DYNAMIC_ADD:
                return False
        except KeyError:
            # skip the treatment if tagDate not exist
            print(u"{} 于 {} 填写的病程 tagDate 为空，请检查！".format(self.patient_name, self.expected))
            return False

        fixed = False
        for item in self.data[KEY_CHECKS]:
            if item['item'] in ITEMS.keys():
                if check(item, self.patient_name, self.expected, self.fix):
                    fixed = True

        if self.fix and fixed:
            self.update()

        return fixed

    def update(self):
        """update treatment status from `complete` to `im-progress`"""
        self.data['classType'] = 'in-progress'
        self.data['fix-info'] = 'fix by NDIP checker.'

    def update_enter_date(self, enter_date):
        """update enter date to target enter date.

        return value is whether this Treat has been modified.
        """

        # skip update dynamic added treatments
        if self.data[KEY_TAG_DATE] == VALUE_DYNAMIC_ADD:
            print('动态病程，无需调整。')
            return False

        # get target date
        stage = int(self.data['stage'].split()[0])
        origin = datetime.strptime(enter_date, DATE_FORMAT)
        target = origin + timedelta(weeks=stage)
        target_date = datetime.strftime(target, DATE_FORMAT)

        prompt = "update treatment date from '{}' to '{}'? [Y/n]".format(self.expected, target_date)
        if not command_confirm(prompt):
            print("You've cancelled this update.")
            return False

        # if not started, change actual date
        if not self.data.get(KEY_IS_STARTED, False):
            origin = self.data[KEY_ACTUAL_DATE]
            print("treatment not started, update actual date from '{}' to '{}'".format(origin, target_date))
            self.data[KEY_ACTUAL_DATE] = target_date

        # update expect date
        origin = self.data[KEY_EXPECT_DATE]
        print("update expect date from '{}' to '{}'".format(origin, target_date))
        self.data[KEY_EXPECT_DATE] = target_date

        # update tag date as: stage + ' ' + expected date
        origin = self.data[KEY_TAG_DATE]
        tag_date = self.data['stage'] + ' ' + target_date
        print(u"update tag date from '{}' to '{}'".format(origin, tag_date))
        self.data[KEY_TAG_DATE] = tag_date

        return True


class Treatment:
    """Represents a patient's all treatment record."""

    # TODO: refactor fix to check method
    # TODO: encapsulate patient info to a object
    def __init__(self, json_obj, record_id, patient_id, patient_name=None, fix=False):
        """init treatment."""

        self.record_id = record_id
        self.patient_id = patient_id
        self.patient_name = patient_name
        self.data = json_obj
        self.fix = fix

    def check(self, db):
        fixed = False
        for data in self.data[KEY_TREAT_LIST]:
            treat = Treat(data, self.patient_name, self.fix)
            if treat.check():
                fixed = True

        if self.fix and fixed:
            self.update(db)

        return fixed

    def update(self, db):
        """Update the treatment record to database."""

        prompt = "Do you really want to update patient's changes to DB? [Y/n]"
        if not command_confirm(prompt):
            return False

        data = json.dumps(self.data, ensure_ascii=False)
        sql = "UPDATE hospital_record set treatment=%s where id=%s"
        val = (data, self.record_id)

        cur = db.cursor()
        cur.execute(sql, val)

        db.commit()
        print(cur.rowcount, " record(s) affected in hospital_record")

        return True

    @property
    def enter_date(self):
        """return patient's enter_date."""
        # TODO: need to valid enter_date from hospital_patient too

        return self.data[KEY_TREAT_LIST][0][KEY_EXPECT_DATE]

    def update_enter_date(self, target, patient_id, db):
        """update patient's enter date"""

        # confirm format as YYYY-MM-DD, as format like `2018-3-14` is accepted

        # update each treat
        changed = False
        for data in self.data[KEY_TREAT_LIST]:
            treat = Treat(data)
            if treat.update_enter_date(target):
                changed = True

        # update treatment, besides patient's info also need to update.
        if changed:
            if self.update(db):
                update_patient_enter_date(patient_id, db, target)



# encoding: utf-8


def command_confirm(prompt="Do you want to do the fix? [Y/n]: "):
    """Get confirm from command.
    
     Y|y: returns True
     N|n: returns False"""

    prompt_w = "Can't recognize, please retry. [Y/n]: "

    answer = raw_input(prompt)
    while answer not in ['Y', 'y', 'N', 'n']:
        answer = raw_input(prompt_w)

    if answer in ['Y', 'y']:
        return True
    else:
        return False


if __name__ == '__main__':
    res = command_confirm()
    print(res)



# encoding: utf-8
import sys

from parser_utils import parse_args


def execute_from_command_line():
    """run script"""

    args = parse_args(sys.argv[1:])
    args.func(args)

if __name__ == "__main__":
    execute_from_command_line()



# encoding: utf-8
import json
import MySQLdb

from treatment import Treatment


def get_db(args):
    """get db handler"""

    return MySQLdb.connect(
        host=args.host, db=args.database,
        user=args.user, passwd=args.password,
        charset='utf8')


def handle_check_all(args):
    """check all treatment stictly"""

    print("Not support yet.")


def handler_check_dynamic_treats(args):
    """check dynamic arguments."""
    db = get_db(args)

    cur = db.cursor()
    cur.execute("select id, treatment, patient_id from hospital_record")

    for row in cur.fetchall():
        record_id = row[0]
        data = json.loads(row[1])
        patient_id = row[2]
        cur2 = db.cursor()
        sql = "select code from hospital_patient where id={}".format(patient_id)
        cur2.execute(sql)
        patient_name = cur2.fetchall()[0][0]

        treatment = Treatment(data, record_id, patient_id, patient_name, args.fix)
        treatment.check(db)

    db.close()


def handler_change_enter_date(args):
    """change patient's enter date."""

    db = get_db(args)
    cur = db.cursor()

    # get treatment record
    sql = "SELECT treatment, patient_id FROM hospital_record WHERE id={}".format(args.record_id)
    print(sql)
    cur.execute(sql)
    record = cur.fetchone()
    data = json.loads(record[0])
    patient_id = int(record[1])

    treatment = Treatment(data, args.record_id, patient_id)

    # if enter date not equals to the given `original parameter, quit
    if treatment.enter_date != args.original:
        msg = "original enter date {} not match existing: {}".format(args.original, treatment.enter_date)
        print(msg)

        return False

    treatment.update_enter_date(args.target, patient_id, db)

    db.close()



"""Test cases for argparse."""
import pytest
import unittest

from ndip_checker.parser_utils import get_parser, parse_args


class ParserTest(unittest.TestCase):
    """Test case for `get_parser`"""

    def setUp(self):
        self.parser = get_parser()

    def test_no_argument(self):
        """test no argument was"""

        args = ''.split()
        with pytest.raises(SystemExit) as e:
            self.parser.parse_args(args)

        assert e.type == SystemExit
        assert e.value.code == 2

    def test_check_all_parameter(self):
        """test all parameter"""

        args = 'all'.split()
        parsed = self.parser.parse_args(args)
        self.assertEqual(parsed.host, 'localhost')
        self.assertEqual(parsed.user, 'root')
        self.assertEqual(parsed.password, 'ChangeMe1!')
        self.assertEqual(parsed.database, 'hospitalAdmin')

        with pytest.raises(AttributeError) as e:
            self.assertEqual(parsed.fix, False)

    def test_args_not_match(self):
        """test argument not in the same group"""

        args = 'all -f'.split()
        with pytest.raises(SystemExit) as e:
            self.parser.parse_args(args)
        assert e.type == SystemExit
        assert e.value.code == 2

    def test_check_dynamic_parameter(self):
        """test check dynamic"""

        args = 'dynamic-treat'.split()
        parsed = self.parser.parse_args(args)
        self.assertEqual(parsed.host, 'localhost')
        self.assertEqual(parsed.user, 'root')
        self.assertEqual(parsed.password, 'ChangeMe1!')
        self.assertEqual(parsed.database, 'hospitalAdmin')
        self.assertEqual(parsed.fix, False)

        args = 'dynamic-treat -f'.split()
        parsed = self.parser.parse_args(args)
        self.assertEqual(parsed.host, 'localhost')
        self.assertEqual(parsed.user, 'root')
        self.assertEqual(parsed.password, 'ChangeMe1!')
        self.assertEqual(parsed.database, 'hospitalAdmin')
        self.assertEqual(parsed.fix, True)

        args = 'dynamic-treat --fix'.split()
        parsed = self.parser.parse_args(args)
        self.assertEqual(parsed.host, 'localhost')
        self.assertEqual(parsed.user, 'root')
        self.assertEqual(parsed.password, 'ChangeMe1!')
        self.assertEqual(parsed.database, 'hospitalAdmin')
        self.assertEqual(parsed.fix, True)

    def test_optional_parameter(self):
        """test optional parameters.
        
        -h/--host
        -u/--user
        -p/--password
        -d/--database
        """
        args = '-o 127.0.0.1 all'.split()
        parsed = self.parser.parse_args(args)
        self.assertEqual(parsed.host, '127.0.0.1')
        self.assertEqual(parsed.user, 'root')
        self.assertEqual(parsed.password, 'ChangeMe1!')
        self.assertEqual(parsed.database, 'hospitalAdmin')

        args = '--host 127.0.0.1 all '.split()
        parsed = self.parser.parse_args(args)
        self.assertEqual(parsed.host, '127.0.0.1')
        self.assertEqual(parsed.user, 'root')
        self.assertEqual(parsed.password, 'ChangeMe1!')
        self.assertEqual(parsed.database, 'hospitalAdmin')

        args = '-u yarving all'.split()
        parsed = self.parser.parse_args(args)
        self.assertEqual(parsed.host, 'localhost')
        self.assertEqual(parsed.user, 'yarving')
        self.assertEqual(parsed.password, 'ChangeMe1!')
        self.assertEqual(parsed.database, 'hospitalAdmin')

        args = '--user yarving all'.split()
        parsed = self.parser.parse_args(args)
        self.assertEqual(parsed.host, 'localhost')
        self.assertEqual(parsed.user, 'yarving')
        self.assertEqual(parsed.password, 'ChangeMe1!')
        self.assertEqual(parsed.database, 'hospitalAdmin')

        args = '-p asdfjkl all'.split()
        parsed = self.parser.parse_args(args)
        self.assertEqual(parsed.host, 'localhost')
        self.assertEqual(parsed.user, 'root')
        self.assertEqual(parsed.password, 'asdfjkl')
        self.assertEqual(parsed.database, 'hospitalAdmin')

        args = '--password asdfjkl all'.split()
        parsed = self.parser.parse_args(args)
        self.assertEqual(parsed.host, 'localhost')
        self.assertEqual(parsed.user, 'root')
        self.assertEqual(parsed.password, 'asdfjkl')
        self.assertEqual(parsed.database, 'hospitalAdmin')

        args = '-d hello all'.split()
        parsed = self.parser.parse_args(args)
        self.assertEqual(parsed.host, 'localhost')
        self.assertEqual(parsed.user, 'root')
        self.assertEqual(parsed.password, 'ChangeMe1!')
        self.assertEqual(parsed.database, 'hello')

        args = '--database hello all'.split()
        parsed = self.parser.parse_args(args)
        self.assertEqual(parsed.host, 'localhost')
        self.assertEqual(parsed.user, 'root')
        self.assertEqual(parsed.password, 'ChangeMe1!')
        self.assertEqual(parsed.database, 'hello')

    def test_change_enter_date_parameter(self):
        """change enter date parameter"""

        args = 'change-enter-date'.split()
        with pytest.raises(SystemExit) as e:
            self.parser.parse_args(args)
        assert e.type == SystemExit
        assert e.value.code == 2

        args = 'change-enter-date 1'.split()
        with pytest.raises(SystemExit) as e:
            self.parser.parse_args(args)
        assert e.type == SystemExit
        assert e.value.code == 2

        args = 'change-enter-date 1 2018-03-14'.split()
        with pytest.raises(SystemExit) as e:
            self.parser.parse_args(args)
        assert e.type == SystemExit
        assert e.value.code == 2

        args = 'change-enter-date 1 2018-03-14 2018-04-14'.split()
        parsed = self.parser.parse_args(args)
        self.assertEqual(parsed.record_id, 1)
        self.assertEqual(parsed.original, '2018-03-14')
        self.assertEqual(parsed.target, '2018-04-14')

        args = 'change-enter-date 1 20180314 2018-04-14'.split()
        with pytest.raises(SystemExit):
            self.parser.parse_args(args)

        args = 'change-enter-date 1 2018-03-14 20180414'.split()
        with pytest.raises(SystemExit):
            self.parser.parse_args(args)


class ParseArgsTest(unittest.TestCase):
    """test case for function `parse_args`"""

    def test_no_argument(self):
        """test no argument was"""

        args = ''.split()
        with pytest.raises(SystemExit) as e:
            parse_args(args)

        assert e.type == SystemExit
        assert e.value.code == 2

    def test_check_all_parameter(self):
        """test all parameter"""

        args = 'all'.split()
        parsed = parse_args(args)
        self.assertEqual(parsed.host, 'localhost')
        self.assertEqual(parsed.user, 'root')
        self.assertEqual(parsed.password, 'ChangeMe1!')
        self.assertEqual(parsed.database, 'hospitalAdmin')

        with pytest.raises(AttributeError) as e:
            self.assertEqual(parsed.fix, False)

    def test_args_not_match(self):
        """test argument not in the same group"""

        args = 'all -f'.split()
        with pytest.raises(SystemExit) as e:
            parse_args(args)
        assert e.type == SystemExit
        assert e.value.code == 2

    def test_check_dynamic_parameter(self):
        """test check dynamic"""

        args = 'dynamic-treat'.split()
        parsed = parse_args(args)
        self.assertEqual(parsed.host, 'localhost')
        self.assertEqual(parsed.user, 'root')
        self.assertEqual(parsed.password, 'ChangeMe1!')
        self.assertEqual(parsed.database, 'hospitalAdmin')
        self.assertEqual(parsed.fix, False)

        args = 'dynamic-treat -f'.split()
        parsed = parse_args(args)
        self.assertEqual(parsed.host, 'localhost')
        self.assertEqual(parsed.user, 'root')
        self.assertEqual(parsed.password, 'ChangeMe1!')
        self.assertEqual(parsed.database, 'hospitalAdmin')
        self.assertEqual(parsed.fix, True)

        args = 'dynamic-treat --fix'.split()
        parsed = parse_args(args)
        self.assertEqual(parsed.host, 'localhost')
        self.assertEqual(parsed.user, 'root')
        self.assertEqual(parsed.password, 'ChangeMe1!')
        self.assertEqual(parsed.database, 'hospitalAdmin')
        self.assertEqual(parsed.fix, True)

    def test_optional_parameter(self):
        """test optional parameters.
        
        -h/--host
        -u/--user
        -p/--password
        -d/--database
        """
        args = '-o 127.0.0.1 all'.split()
        parsed = parse_args(args)
        self.assertEqual(parsed.host, '127.0.0.1')
        self.assertEqual(parsed.user, 'root')
        self.assertEqual(parsed.password, 'ChangeMe1!')
        self.assertEqual(parsed.database, 'hospitalAdmin')

        args = '--host 127.0.0.1 all '.split()
        parsed = parse_args(args)
        self.assertEqual(parsed.host, '127.0.0.1')
        self.assertEqual(parsed.user, 'root')
        self.assertEqual(parsed.password, 'ChangeMe1!')
        self.assertEqual(parsed.database, 'hospitalAdmin')

        args = '-u yarving all'.split()
        parsed = parse_args(args)
        self.assertEqual(parsed.host, 'localhost')
        self.assertEqual(parsed.user, 'yarving')
        self.assertEqual(parsed.password, 'ChangeMe1!')
        self.assertEqual(parsed.database, 'hospitalAdmin')

        args = '--user yarving all'.split()
        parsed = parse_args(args)
        self.assertEqual(parsed.host, 'localhost')
        self.assertEqual(parsed.user, 'yarving')
        self.assertEqual(parsed.password, 'ChangeMe1!')
        self.assertEqual(parsed.database, 'hospitalAdmin')

        args = '-p asdfjkl all'.split()
        parsed = parse_args(args)
        self.assertEqual(parsed.host, 'localhost')
        self.assertEqual(parsed.user, 'root')
        self.assertEqual(parsed.password, 'asdfjkl')
        self.assertEqual(parsed.database, 'hospitalAdmin')

        args = '--password asdfjkl all'.split()
        parsed = parse_args(args)
        self.assertEqual(parsed.host, 'localhost')
        self.assertEqual(parsed.user, 'root')
        self.assertEqual(parsed.password, 'asdfjkl')
        self.assertEqual(parsed.database, 'hospitalAdmin')

        args = '-d hello all'.split()
        parsed = parse_args(args)
        self.assertEqual(parsed.host, 'localhost')
        self.assertEqual(parsed.user, 'root')
        self.assertEqual(parsed.password, 'ChangeMe1!')
        self.assertEqual(parsed.database, 'hello')

        args = '--database hello all'.split()
        parsed = parse_args(args)
        self.assertEqual(parsed.host, 'localhost')
        self.assertEqual(parsed.user, 'root')
        self.assertEqual(parsed.password, 'ChangeMe1!')
        self.assertEqual(parsed.database, 'hello')

    def test_change_enter_date_parameter(self):
        """change enter date parameter"""

        args = 'change-enter-date'.split()
        with pytest.raises(SystemExit) as e:
            parse_args(args)
        assert e.type == SystemExit
        assert e.value.code == 2

        args = 'change-enter-date 1'.split()
        with pytest.raises(SystemExit) as e:
            parse_args(args)
        assert e.type == SystemExit
        assert e.value.code == 2

        args = 'change-enter-date 1 2018-03-14'.split()
        with pytest.raises(SystemExit) as e:
            parse_args(args)
        assert e.type == SystemExit
        assert e.value.code == 2

        args = 'change-enter-date 1 2018-03-14 2018-04-14'.split()
        parsed = parse_args(args)
        self.assertEqual(parsed.record_id, 1)
        self.assertEqual(parsed.original, '2018-03-14')
        self.assertEqual(parsed.target, '2018-04-14')

        args = 'change-enter-date 1 2018-3-14 2018-4-14'.split()
        parsed = parse_args(args)
        self.assertEqual(parsed.record_id, 1)
        self.assertEqual(parsed.original, '2018-03-14')
        self.assertEqual(parsed.target, '2018-04-14')



# encoding: utf-8


def update_patient_enter_date(patient_id, db, enter_date):
    """update patient's information."""

    sql = "UPDATE hospital_patient set enter_date=%s where id=%s"
    val = (enter_date, patient_id)

    cur = db.cursor()
    cur.execute(sql, val)

    db.commit()
    print(cur.rowcount, " record(s) affected in hospital_patient")

    return True



import pytest
from unittest import TestCase

from ndip_checker.treatment import Treatment


class TreatmentTest(TestCase):
    """Test case for Treatment"""

    def setUp(self):
        mock_data = {'a': 1}
        self.treatment = Treatment(mock_data, 1, 1)



# encoding: utf-8
import json
import MySQLdb


def get_db(args):
    """return a db object based on parameters in args."""

    return MySQLdb.connect(
        host=args.host, db=args.database,
        user=args.user, passwd=args.password,
        charset='utf8')


def get_all_objects(args):
    pass
